/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Weapon bazooka : projette une roquette avec un angle et une force donne.
 *****************************************************************************/

#ifndef BAZ_TETE_C_H
#define BAZ_TETE_C_H
//-----------------------------------------------------------------------------
#include "../include/base.h"
#include "../weapon/weapon.h"
#include "../gui/progress_bar.h"
#include "../object/physical_obj.h"
#ifdef CL
#include <ClanLib/display.h>
#endif
//-----------------------------------------------------------------------------
namespace Wormux {
//-----------------------------------------------------------------------------

// Roquette du bazooka  tte chercheuse
class RoquetteTeteCherche : public WeaponProjectile
{
protected:
  double angle_local;
  double temps_debut_tir;
#ifdef CL
  CL_Point m_cible;
#else
  Point2i m_cible;
#endif
  bool m_attire;
public:
  RoquetteTeteCherche();
  void Tire (double force, int x, int y, uint cible_x,uint cible_y);
  uint ChoixFrame(double angle);
  void Init();
  void Reset();
  void Refresh();
protected:
  void SetTarget (int x,int y);
  void SignalCollision();
};

//-----------------------------------------------------------------------------

class AutomaticBazooka : public Weapon
{
private:

  struct s_cible
  {
#ifdef CL
    CL_Point pos;
#else
    Point2i pos;
#endif
    bool choisie;
#ifdef CL
    CL_Surface image;
#else
    SDL_Surface *image;
#endif
  } cible;
  
  void p_Init();
  bool p_Shoot();

public:
#ifdef CL
  CL_Surface impact;    // Image (alpha) de l'impact
#else
  SDL_Surface *impact;
#endif
  RoquetteTeteCherche roquette;

  AutomaticBazooka();
  void Draw ();
  void Refresh();
  void p_Select(); 

  bool IsReady() const;

  // Le bazooka explose car il a t pouss  bout !
  void ExplosionDirecte();

  virtual void ChooseTarget();
  void DrawTarget();

  ExplosiveWeaponConfig& cfg();

protected:
  void Explosion();
};

extern AutomaticBazooka auto_bazooka;
//-----------------------------------------------------------------------------
} // namespace Wormux
#endif
